
ExcludeArch: %{ix86}

Name:			gpaw
Version:		23.9.1
Release:    2%{?dist}
Summary:		A grid-based real-space PAW method DFT code

License:		GPLv3+
URL:			https://wiki.fysik.dtu.dk/gpaw/
Source0:		https://gitlab.com/%{name}/%{name}/-/archive/%{version}/%{name}-%{version}.tar.gz
Source1:		gpaw


BuildRequires:		time
BuildRequires:		libxc-devel
BuildRequires:		flexiblas-devel
BuildRequires:		fftw-devel

BuildRequires:		python3-devel
BuildRequires:		python3-pytest
BuildRequires:		python3-scipy
BuildRequires:		python3-setuptools

BuildRequires:		python3-ase


BuildRequires:		%{name}-setups


%global desc_base \
GPAW is a density-functional theory (DFT) Python code based\
on the projector-augmented wave (PAW) method and the\
atomic simulation environment (ASE). It uses real-space uniform grids and\
multigrid methods or atom-centered basis-functions.

%global _description\
%{desc_base}

%description %_description

%package common
Summary:		%{name} - common files
Requires:		%{name}-setups
BuildArch:		noarch

%description common
%{desc_base}

This package contains the common data files.


%package -n python3-%{name}
Summary:		A grid-based real-space PAW method DFT code for Python 3
Requires:		python3-ase
Requires:		python3-scipy
Requires:		%{name}-common = %{version}-%{release}

%description -n python3-%{name}
%{desc_base}

%package -n python3-%{name}-openmpi
Summary:		python3-%{name} - openmpi version
BuildRequires:		openssh-clients
BuildRequires:		openmpi-devel
BuildRequires:		scalapack-openmpi-devel
Requires:		%{name}-common = %{version}-%{release}

%description -n python3-%{name}-openmpi
%{desc_base}

This package contains the openmpi Python 3 version.

%package -n python3-%{name}-mpich
Summary:		python3-%{name} - mpich version
BuildRequires:		mpich-devel
BuildRequires:		scalapack-mpich-devel
Requires:		%{name}-common = %{version}-%{release}

%description -n python3-%{name}-mpich
%{desc_base}

This package contains the mpich Python 3 version.


%prep
%setup -qTc -a 0
pushd %{name}-%{version}
popd

mv %{name}-%{version} python3

cp python3/siteconfig_example.py python3/siteconfig.py

cp -p python3/LICENSE .

find python3/tools -type f | xargs sed -i '1s|^#!/usr/bin/env python.*|#!%{_bindir}/python3|'

sed -i 's/except AttributeError:/except (AttributeError, AssertionError):/' python3/gpaw/test/conftest.py


%build
%set_build_flags

%global dobuild() \
cat siteconfig.py \
unset CC \
${PYTHON} setup.py build && \
mv build build$MPI_SUFFIX && \
${PYTHON} setup.py clean

sed -i 's/.*mpi =.*/mpi = False/' python3/siteconfig.py
sed -i 's/.*scalapack =.*/scalapack = False/' python3/siteconfig.py
echo "libraries += ['flexiblas']" >> python3/siteconfig.py
echo "extra_compile_args += ['-fPIC']" >> python3/siteconfig.py
echo "import os" >> python3/siteconfig.py
echo "include_dirs += [os.environ.get('MPI_INCLUDE', '/usr/include')]" >> python3/siteconfig.py

pushd python3
MPI_SUFFIX=_serial PYTHON=python3 %dobuild
popd

%{_openmpi_load}
sed -i 's/.*mpi =.*/mpi = True/' python3/siteconfig.py
sed -i 's/.*scalapack =.*/scalapack = True/' python3/siteconfig.py
sed -i 's/# compiler =.*/compiler = "mpicc"/' python3/siteconfig.py
which mpicc
mpicc --version
mpicc foo.c --showme
pushd python3
PYTHON=python3 %dobuild
popd
%{_openmpi_unload}

%{_mpich_load}
sed -i 's/.*mpi =.*/mpi = True/' python3/siteconfig.py
sed -i 's/.*scalapack =.*/scalapack = True/' python3/siteconfig.py
sed -i 's/# compiler =.*/compiler = "mpicc"/' python3/siteconfig.py
which mpicc
mpicc --version
mpicc -compile_info
mpicc -link_info
pushd python3
PYTHON=python3 %dobuild
popd
%{_mpich_unload}


%install

%global doinstall() \
mkdir -p %{buildroot}/$MPI_BIN&& \
install -p -m 755 build$MPI_SUFFIX/scripts-*/* %{buildroot}/$MPI_BIN/&& \
install -p -m 755 %{SOURCE1} %{buildroot}/$MPI_BIN/&& \
mkdir -p %{buildroot}/$MPI_PYTHON3_SITEARCH&& \
cp -rpv build$MPI_SUFFIX/lib.*/%{name} %{buildroot}/$MPI_PYTHON3_SITEARCH/&& \
install -p -m 755 build$MPI_SUFFIX/lib.*/*.so %{buildroot}/$MPI_PYTHON3_SITEARCH/

pushd python3
PYTHON=python3 MPI_SUFFIX="_serial" MPI_BIN=%{_bindir} MPI_PYTHON3_SITEARCH=%{python3_sitearch} %doinstall
popd

%{_openmpi_load}
pushd python3
PYTHON=python3 %doinstall
popd
%{_openmpi_unload}

%{_mpich_load}
pushd python3
PYTHON=python3 %doinstall
popd
%{_mpich_unload}


%check

export NPROC_PARALLEL=2 # test on 4 cores (scalapack test needs that)

export TIMEOUT_OPTS='--preserve-status --kill-after 10 1800'

%global docheck() \
GPAW_PLATFORM_GLOB=$($PYTHON -c "from distutils import util, sysconfig; print(util.get_platform()+'-*')")&& \
export PYTHONPATH=$(echo $(pwd)/build$MPI_SUFFIX/lib.${GPAW_PLATFORM_GLOB}) \
PATH=`pwd`/tools:${PATH} \
timeout ${TIMEOUT_OPTS} time $GPAW_EXECUTABLE -m ci -v 2>&1 | tee gpaw-test${NPROC}$MPI_SUFFIX.log || true

pushd python3
MPI_SUFFIX="_serial" PYTHON="python3" GPAW_EXECUTABLE="pytest" NPROC=1 %docheck
popd

%{_openmpi_load}
pushd python3
PYTHON="python3" GPAW_EXECUTABLE="mpiexec -np ${NPROC_PARALLEL} pytest" NPROC=${NPROC_PARALLEL} %docheck
popd
%{_openmpi_unload}

%{_mpich_load}
pushd python3
PYTHON="python3" GPAW_EXECUTABLE="mpiexec -np ${NPROC_PARALLEL} pytest" NPROC=${NPROC_PARALLEL} %docheck
popd
%{_mpich_unload}


%files common
%doc LICENSE
%{_bindir}/%{name}*


%files -n python3-%{name}
%exclude %{_bindir}/%{name}*
%{python3_sitearch}/%{name}
%{python3_sitearch}/_%{name}*.so


%files -n python3-%{name}-openmpi
%exclude %{_libdir}/openmpi/bin
%{python3_sitearch}/openmpi/%{name}
%{python3_sitearch}/openmpi/_%{name}*.so


%files -n python3-%{name}-mpich
%exclude %{_libdir}/mpich/bin
%{python3_sitearch}/mpich/%{name}
%{python3_sitearch}/mpich/_%{name}*.so


%changelog
* Tue May 07 2024 Fanjun Kong <fanjunkong@tencent.com> - 23.9.1-2
- epol debrand

* Fri Apr 12 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 23.9.1-1
- initial build
